@@ -54,13 +54,13 @@
 #define ISDIGIT(c) ((unsigned int) (c) - '0' <= 9)
 
 /* Macro for validation of range components. */
-#define VALID_RANGE(range) (ISDIGIT (range) || range == '-' || range == '+')
+#define VALID_RANGE(range) (ISDIGIT (range) || (range) == '-' || (range) == '+')
 
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "linecut"
 
 /* The version of this program (floating-point value). */
-#define PROGRAM_VERSION 0.4
+#define PROGRAM_VERSION 0.5
 
 /* The authors of this program. */
 #define AUTHORS "Steven Schubiger"
@@ -69,7 +69,7 @@
 #define BUGS_MAIL_ADDRESS "stsc@members.fsf.org"
 
 /* Maximum of allowed range sets. */
-#define MAX_RANGE_SETS (16 * 2)
+#define MAX_RANGE_SETS 16
 
 /* If true, argument to range parameter provided.  */
 static bool range_mode;
@@ -99,20 +99,20 @@ static char *program_name;
 static int range_max;
 
 /* The sets of ranges. */
-static long ranges[MAX_RANGE_SETS];
+static long ranges[MAX_RANGE_SETS * 2];
 
 /* The file offsets to seek for. */
 static off_t offsets[MAX_RANGE_SETS];
 
 /* How many lines the stream consists of. */
-static int lines;
+static uintmax_t lines;
 
 /* Container for file-descriptor and handle. */
 static struct 
   {
     int fd;
     FILE *fh;
-  }
+  } 
 input;
 
 /* Buffer for line numbers.
@@ -257,7 +257,7 @@ next_line_num (void)
 static bool
 elide_lines_seekable (const char *filename, int fd,
                       uintmax_t line_from, uintmax_t line_to,
-                      off_t offset_from, off_t offset_to)
+                      off_t offset)
 {
   char buffer[BUFSIZ];
   char *buf, *eol;
@@ -266,7 +266,7 @@ elide_lines_seekable (const char *filename, int fd,
 
   seen_complete_line = true;
 
-  if ((lseek (fd, offset_from, SEEK_SET)) < 0)
+  if ((lseek (fd, offset, SEEK_SET)) < 0)
     {
       error (0, errno, _("cannot lseek %s"), quote (filename));
       return false;
@@ -290,7 +290,7 @@ elide_lines_seekable (const char *filename, int fd,
         {
           if (bytes_to_write)
             {
-              /* full line with trailing newline detected. */
+              /* Full line with trailing newline detected. */
               if (eol = memchr (buf, '\n', bytes_to_write))
                 {
                   if (number_mode && seen_complete_line)
@@ -319,7 +319,7 @@ elide_lines_seekable (const char *filename, int fd,
                   bytes_to_write -= eol - buf + 1; /* + 1 for subtracting the newline too. */
                   buf = eol + 1; /* + 1 because we advance beyond the end-of-line. */
                 }
-              /* part of line with no trailing newline. */
+              /* Part of line with no trailing newline. */
               else
                 {
                   if (number_mode)
@@ -359,16 +359,16 @@ static bool
 range_lines (const char *filename, int fd)
 {
   bool ok;
-  int i;
+  int c, i;
   long line_from, line_to;
-  off_t offset_from, offset_to;
+  off_t offset;
 
   ok = true;
 
   if (print_headers)
     write_header (filename);
 
-  for (i = 0; i < range_max; i += 2)
+  for (c = 0, i = 0; i <= range_max; c++, i += 2)
     {
       line_from = ranges[i];
       line_to = ranges[i+1];
@@ -376,10 +376,9 @@ range_lines (const char *filename, int fd)
       relative_range_to_absolute (&line_from);
       relative_range_to_absolute (&line_to);
 
-      offset_from = offsets[i];
-      offset_to = offsets[i+1];
+      offset = offsets[c];
 
-      ok &= elide_lines_seekable (filename, fd, line_from, line_to, offset_from, offset_to);
+      ok &= elide_lines_seekable (filename, fd, line_from, line_to, offset);
     }
   return ok;
 }
@@ -388,19 +387,17 @@ static bool
 determine_seek_offsets (const char *filename, int fd)
 {
   char buffer[BUFSIZ];
-  int i;
-  long line_iter, range_abs, range_end;
+  int c, i;
+  long line_iter, range_abs;
   size_t bytes_offset;
   off_t seek_offset;
 
+  c = 0;
   i = 0;
   bytes_offset = 0;
   line_iter = 0;
   seek_offset = 0;
 
-  range_end = ranges[range_max];
-  relative_range_to_absolute (&range_end);
-
   if ((lseek (fd, 0, SEEK_SET)) < 0)
     {
       error (0, errno, _("cannot lseek %s"), quote (filename));
@@ -432,14 +429,15 @@ determine_seek_offsets (const char *filename, int fd)
       range_abs = ranges[i];
       relative_range_to_absolute (&range_abs);
       if (line_iter == range_abs)
-        offsets[i++] = seek_offset;
-
-      /* We don't need to determine seek offsets past the specified final
-         ending range, which is always guaranteed to be the last line.
-         This will mostly avoid a tremendous speed impact, when input is huge.  */
-      if (line_iter == range_end)
+        {
+          offsets[c++] = seek_offset;
+          i += 2;
+        }
+      /* We do not need to determine seek offsets past the last starting 
+         range line. This will mostly avoid a tremendous speed impact,
+         when input is huge.  */
+      if (i > range_max)
         break;
-
       if ((seek_offset = lseek (fd, bytes_offset, SEEK_SET)) < 0)
         {
           error (0, errno, _("cannot lseek %s"), quote (filename));
@@ -495,7 +493,7 @@ validate_ranges (const char *filename, int fd)
 
   ok = count_lines_in_stream (filename, fd);
 
-  for (i = 0; i < range_max; i += 2) 
+  for (i = 0; i <= range_max; i += 2) 
     {
       line_start = ranges[i];
       line_end = ranges[i+1];
@@ -669,10 +667,7 @@ parse_range_sets (char *ranges_arg)
           buf = xmalloc (digits_count * sizeof (char));
           buf_orig = buf;
           while (VALID_RANGE (*ranges_arg))
-            {
-              *buf = *ranges_arg++;
-              buf++;
-            }
+            *buf++ = *ranges_arg++;
           *buf = '\0';
           buf = buf_orig;
 
@@ -694,8 +689,8 @@ parse_range_sets (char *ranges_arg)
           range_max++;
           free (buf);
 
-          if (range_max > MAX_RANGE_SETS)
-            error (EXIT_FAILURE, 0, _("only %d sets permitted"), MAX_RANGE_SETS / 2);
+          if (range_max > (MAX_RANGE_SETS * 2))
+            error (EXIT_FAILURE, 0, _("only %d sets permitted"), MAX_RANGE_SETS);
 
           ranges_seen++;
         }
